from os import listdir
from os.path import join

from pygame.mixer import Channel, Sound, music, find_channel, get_num_channels

from GameChild import *
from Input import *

class Audio(GameChild):

    UP, DOWN = .1, -.1
    BASE_VOLUME = .8

    def __init__(self, game):
        GameChild.__init__(self, game)
        self.original_volumes = {}
        self.volume = self.BASE_VOLUME
        if self.check_command_line("-mute"):
            self.volume = 0
        self.subscribe(self.respond)

    def set_volume(self, volume=None, increment=None, mute=False):
        if mute:
            self.volume = 0
        elif increment:
            self.volume += increment
            if self.volume > 1:
                self.volume = 1.0
            elif self.volume < 0:
                self.volume = 0
        else:
            self.volume = volume

    def respond(self, event):
        compare = self.get_game().delegate.compare
        if compare(event, "volume-mute"):
            self.set_volume(mute=True)
        elif compare(event, "volume-up"):
            self.set_volume(increment=self.UP)
        elif compare(event, "volume-down"):
            self.set_volume(increment=self.DOWN)

    def update(self):
        for ii in xrange(get_num_channels()):
            channel = Channel(ii)
            sound = channel.get_sound()
            if sound is not None:
                if sound not in self.original_volumes.keys():
                    self.original_volumes[sound] = sound.get_volume()
                sound.set_volume(self.original_volumes[sound] * self.volume)
